import React, { PureComponent } from 'react';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Table, Form, Spin, Input, InputNumber, Button, Icon, message } from 'antd';
import queryString from 'query-string';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import ProductSelectModal from './product';
import { Codes, pageSize } from '../../../utils/config';

@Form.create()
@connect(state => ({
  comboDetail: state.comboDetail,
  product: state.product,
}))
export default class PackageProfile extends PureComponent {

  handleEditProuctClick = () => {
    this.props.dispatch({ type: 'comboDetail/showModal' });
    this.props.dispatch({
      type: 'product/query',
      payload: {
        type: Codes.CODE_COURSE,
        status: Codes.CODE_NORMAL,
        pageNo: 1,
        pageSize,
      }
    });
  }

  handleInputNumberChange = (value, record) => {
    const { currentItem } = this.props.comboDetail;
    const { products } = currentItem;
    const boundPriceProducts = [...products];
    boundPriceProducts.map(item => item.pid === record.pid ? item.cpPrice = value : null);
    this.props.dispatch({
      type: 'comboDetail/savePrice',
      payload: boundPriceProducts,
    });
  }

  handleProductModalOk = (data) => {
    this.props.dispatch({
      type: 'comboDetail/saveProducts',
      payload: data ,
    });
  }

  handleProductModalCancel = () => {
    this.props.dispatch({ type: 'comboDetail/hideModal' });
  }

  handleProductModalSearch = (data) => {
    const newData = { ...data };
    if (newData.keyword) {
      newData[newData.field] = newData.keyword;
    }
    delete newData.field;
    delete newData.keyword;
    this.props.dispatch({
      type: 'product/query',
      payload: {
        type: Codes.CODE_COURSE,
        status: Codes.CODE_NORMAL,
        pageNo: 1,
        pageSize,
      }
    });
  }

  handleProductModalTableChange = (pagination, filterArgs, filters) => {
    const newFilters = { ...filters };
    if (newFilters.keyword) {
      newFilters[newFilters.field] = newFilters.keyword;
    }
    delete newFilters.field;
    delete newFilters.keyword;

    const data = {
      ...newFilters,
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      status: Codes.CODE_NORMAL,
    };

    Object.keys(data).map(key => data[key] ? null : delete data[key]);
    dispatch({ type: 'product/query', payload: data });
  }

  handlePageCancel = () => {
    const { filters } = this.props.comboDetail;
    this.props.dispatch(routerRedux.push({
      pathname: '/product/package',
      search: queryString.stringify(filters),
    }));
  }

  handlePageSubmit = (e) => {
    e.preventDefault();
    const { dispatch, form, comboDetail } = this.props;
    const { currentItem, operType, filters } = comboDetail;
    const { products } = currentItem;
    const { validateFields, getFieldsValue } = form;
    validateFields((errors) => {
      if (errors) return;
      if (products && products.filter(item => !item.cpPrice).length > 0 ) {
        message.error('还有供应商价格未填写!');
        return;
      }
      // add params `code` and `name`
      const data = { ...getFieldsValue() };
      // add params `products`
      if (Array.isArray(products)) {
        data.products = products.map(item => ({
          pid: item.pid,
          type: item.type,
          cpPrice: item.cpPrice,
        }));
      }
      // add params `status`
      if (operType === 'create') {
        data.status = Codes.CODE_NORMAL;
      }
      else if (operType === 'update') {
        data.status = currentItem.status;
        data.id = currentItem.id;
      }
      dispatch({
        type: `comboDetail/${operType}`,
        payload: data,
        callback: () => {
          dispatch(routerRedux.push({
            pathname: '/product/package',
            search: queryString.stringify(filters),
          }));
        }
      });
    });
  }

  render() {
    const { form, comboDetail, product } = this.props;
    const { modalShow, currentItem, itemLoading } = comboDetail;
    const { list, listLoading, pagination } = product;
    const { products } = currentItem;
    const { getFieldDecorator } = form;

    const formItemLayout = {
      labelCol: { span: 7 },
      wrapperCol: { span: 12 },
    };

    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    const tableFormLayout = {
      wrapperCol: { offset: 7, span: 12 },
    };

    const columns = [{
      title: '产品封面',
      dataIndex: 'coverUrl',
      key: 'coverUrl',
      render: (text, record) => {},
    },{
      title: '产品编号',
      dataIndex: 'code',
      key: 'code',
    },{
      title: '产品名称',
      dataIndex: 'name',
      key: 'name',
    },{
      title: '供应商',
      dataIndex: 'cpName',
      key: 'cpName',
    },{
      title: '供应商价格',
      dataIndex: 'cpPrice',
      key: 'cpPrice',
      render: (text, record) => (
        <InputNumber
          min={0}
          value={record.cpPrice}
          onChange={(value) => this.handleInputNumberChange(value, record)}
          formatter={value => `¥ ${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}
          parser={value => value.replace(/\¥\s?|(,*)/g, '')}
        />
      ),
    }];

    return (
      <PageHeaderLayout>
        <Spin spinning={itemLoading}>
          <Card>
            <Form layout="horizontal" onSubmit={this.handlePageSubmit}>
              <Form.Item label="产品包编号" { ...formItemLayout }>
                {getFieldDecorator('code', {
                  rules: [{ required: true, type: 'string', message: '编号为必填项!' }],
                })(<Input placeholder="请输入" />)}
              </Form.Item>
              <Form.Item label="产品包名称" { ...formItemLayout }>
                {getFieldDecorator('name', {
                  rules: [{ required: true, type: 'string', message: '名称为必填项!' }],
                })(<Input placeholder="请输入" />)}
              </Form.Item>
              <Form.Item label="选择产品" { ...formItemLayout }>
                <Button onClick={this.handleEditProuctClick} type="primary" size="small" icon="select">选择</Button>
              </Form.Item>
              <Form.Item { ...tableFormLayout }>
                <Table
                  simple
                  bordered
                  columns={columns}
                  dataSource={products || []}
                  pagination={false}
                  rowKey={record => record.id}
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该产品包内暂无相关产品!</span>
                  }}
                />
              </Form.Item>
              <Form.Item { ...submitFormLayout } style={{ marginTop: 32 }}>
                <Button onClick={this.handlePageCancel}>取消</Button>
                <Button type="primary" style={{ marginLeft: 35 }} htmlType="submit">提交</Button>
              </Form.Item>
            </Form>
            {/*产品选择模态框*/}
            <ProductSelectModal
              rowKeyName="id"
              modalVisible={modalShow}
              selTableData={products || []}
              style={{ top: 20 }}
              fsTableDataSource={list}
              fsTableLoading={listLoading}
              fsTablePagination={pagination}
              onOk={this.handleProductModalOk}
              onCancel={this.handleProductModalCancel}
              onSearch={this.handleProductModalSearch}
              fsTableOnChange={this.handleProductModalTableChange}
            />
          </Card>
        </Spin>
      </PageHeaderLayout>
    );
  }
}
